// SharedMemDLL.cpp : Defines the exported functions for the DLL application.
// Use Example.vi and SharedMem.exe to show how to communicate between a C/C++ application and 
// LabVIEW.
// Some disclaimers:
// 1) You can only have 1 side use the DLL. I'm not sure why. So, since LabVIEW needs the DLL, let
// LabVIEW use the DLL and use the direct calls in the C program as in SharedMem
// 2) 

#include "stdafx.h"
#include <windows.h>
#include <stdio.h>
#include "SharedMemDLL.h"
#include "..\SharedMem\privilege.h"
#include "..\SharedMem\SharedMem.h"

// #define LOGGING
static HANDLE hMapFile1 ;
static HANDLE ghMutex1 ;
static HANDLE *hMapFile = &hMapFile1 ;
static HANDLE *ghMutex = &ghMutex1 ;
static int *pQueue1 ;
static int **pQueue = &pQueue1 ;
TCHAR fileMapName[]=TEXT(FILE_MAP_NAME);
TCHAR mutexName[]=TEXT(MUTEX_NAME) ;


#ifdef __cplusplus
	extern "C" {
#endif
double add(double a, double b)
{
	return a + b ;
}
#ifdef __cplusplus
}
#endif


#ifdef __cplusplus
	extern "C" {
#endif
int InitSharedMem(int createMap, /* char *fileMapName, char *mutexName, */ int elementSize, int maxElements /* , HANDLE *hMapFile, HANDLE *ghMutex, int **pQueue */ )
{
	// createMap == 1 - Shared Memory of fileMapSize, == 0 - just reference it
	// fileMapName - Name of Shared Memory - must start with "Global\\"
	// mutexName - Name of Mutex - must start with "Global\\"

	// On completions, hMapFile and ghMutex point to the HANDLE to the Shared Memory and to the HANDLE to the Mutex.
	// pQueue points to the start of the Shared Memory

	// Returns 0 on no error, getLastError() on error

	// Create Mutex
#ifdef LOGGING
	FILE *logfile ;
	logfile = fopen("c:\\temp\\InitSharedMem.txt","ab") ;
#endif

	DWORD dwWaitResult; 
	int i ;
#ifdef LOGGING
	fprintf(logfile,"Start\n") ;
	fclose(logfile) ;
	logfile = fopen("c:\\temp\\InitSharedMem.txt","ab") ;
#endif

	*ghMutex = CreateMutex( 
        NULL,              // default security attributes
        FALSE,             // initially not owned
        mutexName);             // unnamed mutex

#ifdef LOGGING
	fprintf(logfile,"*ghMutex:%016x\n", *ghMutex) ;
	fclose(logfile) ;
	logfile = fopen("c:\\temp\\InitSharedMem.txt","ab") ;
#endif

	if (*ghMutex == NULL) 
    {
		return GetLastError();
    }

	// Upgrade privilege
	Privilege(TEXT("SeLockMemoryPrivilege"), TRUE);


	long largePageMin = (long)GetLargePageMinimum();
	long smSize = ((elementSize*maxElements+HEADER_SIZE)/largePageMin + 1)*largePageMin ;
	// Create File Map
	if (createMap)
	{
		// Create Shared Memory
		*hMapFile = CreateFileMapping(
					INVALID_HANDLE_VALUE,    // use paging file
					NULL,                    // default security
					PAGE_READWRITE | SEC_COMMIT | SEC_LARGE_PAGES,          // read/write access and large pages
					0, // maximum object size (high-order DWORD)
					smSize, // maximum object size (low-order DWORD)
					fileMapName);                 // name of mapping object

	}
	else
	{
		// Reference Shared Memory if already created
		*hMapFile = OpenFileMapping(
					FILE_MAP_ALL_ACCESS,   // read/write access
					FALSE,                 // do not inherit the name
					fileMapName);               // name of mapping object
	}
#ifdef LOGGING
	fprintf(logfile,"*hMapFile:%016x\n", *hMapFile) ;
	fclose(logfile) ;
	logfile = fopen("c:\\temp\\InitSharedMem.txt","ab") ;
#endif
	if (*hMapFile == NULL)
	{
		return GetLastError();
	}

	*pQueue = (int *)MapViewOfFile(*hMapFile, // handle to map object
			FILE_MAP_ALL_ACCESS,  // read/write permission
			0,
			0,
			smSize);
#ifdef LOGGING
	fprintf(logfile,"*pQueue:%016x\n", *pQueue) ;
	fclose(logfile) ;
	logfile = fopen("c:\\temp\\InitSharedMem.txt","ab") ;
#endif
	if (*pQueue == NULL)
	{
		CloseHandle(*hMapFile);
		return GetLastError();
	}
	if (createMap)
	{
		// Initialize Shared Memory
		dwWaitResult = WaitForSingleObject( 
			*ghMutex,    // handle to mutex
			INFINITE);  // no time-out interval
#ifdef LOGGING
		fprintf(logfile,"dwWaitResult:%d\n", dwWaitResult) ;
		fclose(logfile) ;
		logfile = fopen("c:\\temp\\InitSharedMem.txt","ab") ;
#endif
		for (i=0;i<smSize;i++)
		{
			*((char *)(*pQueue) + i) = 0 ;
		}
		MM_VALUE((*pQueue),ELEMENT_SIZE) = elementSize ;
		MM_VALUE((*pQueue),MAX_ELEMENTS) = maxElements ;
		if (!ReleaseMutex(*ghMutex))
		{
			return GetLastError();
		}
	}
#ifdef LOGGING
	fprintf(logfile,"Done\n") ;
	fclose(logfile) ;
#endif
	return(0) ;
}
#ifdef __cplusplus
}
#endif
#ifdef __cplusplus
	extern "C" {
#endif


int ReadSharedMem(/* HANDLE *hMapFile, HANDLE *ghMutex, int **pQueue, */ char *pDst, int *validRead)
{
	// Returns 1 on successful read. 0 if queue is empty or on error
	// Check errorCode for error value
#ifdef LOGGING
	FILE *logfile ;
	logfile = fopen("c:\\temp\\ReadSharedMem.txt","ab") ;
#endif
	DWORD dwWaitResult; 
	int numElements, rdPtr, wrPtr, elementSize, maxElements ;
	int retVal ;
	char *pSrc ;
	*validRead = 0 ;
	retVal = 0 ;
#ifdef LOGGING
	fprintf(logfile,"Start\n") ;
	fclose(logfile) ;
	logfile = fopen("c:\\temp\\ReadSharedMem.txt","ab") ;
#endif

	dwWaitResult = WaitForSingleObject( 
					*ghMutex,    // handle to mutex
					INFINITE);  // no time-out interval
#ifdef LOGGING
		fprintf(logfile,"dwWaitResult:%d\n", dwWaitResult) ;
		fprintf(logfile,"*ghMutex:%016xd\n", *ghMutex) ;
		fclose(logfile) ;
		logfile = fopen("c:\\temp\\ReadSharedMem.txt","ab") ;
#endif
	if (dwWaitResult)
		return(dwWaitResult) ;

	numElements = MM_VALUE((*pQueue),NUM_ELEMENTS) ;
	if (numElements)
	{
		//rdPtr = MM_VALUE((*pQueue),RD_PTR) ;
		//elementSize = MM_VALUE((*pQueue),ELEMENT_SIZE) ;
		//maxElements = MM_VALUE((*pQueue),MAX_ELEMENTS) ;
		//pSrc = (char *)(*pQueue) + HEADER_SIZE + rdPtr*elementSize ;
		//CopyMemory((PVOID)pDst, (PVOID)pSrc, elementSize);
		//MM_VALUE((*pQueue),RD_PTR) = (rdPtr + 1) % maxElements ;
		//MM_VALUE((*pQueue),NUM_ELEMENTS) = numElements - 1 ;
		//*validRead = 1 ;


		wrPtr = MM_VALUE((*pQueue),WR_PTR) ;
		maxElements = MM_VALUE((*pQueue),MAX_ELEMENTS) ;
		if (!wrPtr)
			rdPtr = maxElements -1 ;
		else
			rdPtr = MM_VALUE((*pQueue),WR_PTR) - 1 ;
		elementSize = MM_VALUE((*pQueue),ELEMENT_SIZE) ;
		pSrc = (char *)(*pQueue) + HEADER_SIZE + rdPtr*elementSize ;
		CopyMemory((PVOID)pDst, (PVOID)pSrc, elementSize);
		MM_VALUE((*pQueue),NUM_ELEMENTS) = 0 ;
		*validRead = 1 ;



#ifdef LOGGING
		fprintf(logfile,"numElements:%016x\n",numElements) ;
		fprintf(logfile,"maxElements:%016x\n",maxElements) ;
		fprintf(logfile,"pSrc:%016x\n",pSrc) ;
		fclose(logfile) ;
		logfile = fopen("c:\\temp\\ReadSharedMem.txt","ab") ;
#endif
	}
	if (!ReleaseMutex(*ghMutex))
	{
		retVal = GetLastError();
	}
#ifdef LOGGING
	fprintf(logfile,"Done\n") ;
	fclose(logfile) ;
#endif
	return(retVal) ;
}
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
	extern "C" {
#endif
int WriteSharedMem(/* HANDLE *hMapFile, HANDLE *ghMutex, int **pQueue, */ char *pSrc, int *validWrite)
{
	// Returns 1 if successful write. 0 if queue is full or on error
	// Check errorCode for error value
#ifdef LOGGING
	FILE *logfile ;
	logfile = fopen("c:\\temp\\WriteSharedMem.txt","ab") ;
#endif
	DWORD dwWaitResult; 
	int rdPtr, wrPtr, numElements, elementSize, maxElements ;
	int retVal ;
	char *pDst ;
	*validWrite = 0 ;
	retVal = 0 ;
#ifdef LOGGING
	fprintf(logfile,"Start\n") ;
	fclose(logfile) ;
	logfile = fopen("c:\\temp\\WriteSharedMem.txt","ab") ;
#endif

    dwWaitResult = WaitForSingleObject( 
        *ghMutex,    // handle to mutex
        INFINITE);  // no time-out interval
#ifdef LOGGING
		fprintf(logfile,"dwWaitResult:%d\n",dwWaitResult) ;
		fprintf(logfile,"*ghMutex:%016x\n", *ghMutex) ;
		fclose(logfile) ;
		logfile = fopen("c:\\temp\\WriteSharedMem.txt","ab") ;
#endif
	if (dwWaitResult)
		return(dwWaitResult) ;

	rdPtr = MM_VALUE((*pQueue),RD_PTR) ;
	wrPtr = MM_VALUE((*pQueue),WR_PTR) ;
	numElements = MM_VALUE((*pQueue),NUM_ELEMENTS) ;
	elementSize = MM_VALUE((*pQueue),ELEMENT_SIZE) ;
	maxElements = MM_VALUE((*pQueue),MAX_ELEMENTS) ;
#ifdef LOGGING
	fprintf(logfile,"rdPtr:%d\n",rdPtr) ;
	fprintf(logfile,"wrPtr:%d\n",wrPtr) ;
	fprintf(logfile,"numElements:%d\n",numElements) ;
	fprintf(logfile,"elementSize:%d\n",elementSize) ;
	fprintf(logfile,"maxElements:%d\n",maxElements) ;
	fclose(logfile) ;
	logfile = fopen("c:\\temp\\WriteSharedMem.txt","ab") ;
#endif

/*
	if ((wrPtr != rdPtr) || (numElements == 0))
	{
*/
		pDst = (char *)(*pQueue) +  HEADER_SIZE + wrPtr*elementSize ;
		CopyMemory((PVOID)pDst, (PVOID)pSrc, elementSize);
		MM_VALUE((*pQueue),WR_PTR) = (wrPtr + 1) % maxElements ;
		MM_VALUE((*pQueue),NUM_ELEMENTS) = numElements + 1 ;
		*validWrite = 1 ;
#ifdef LOGGING
	fprintf(logfile,"pDst:%016x\n",pDst) ;
	fclose(logfile) ;
	logfile = fopen("c:\\temp\\WriteSharedMem.txt","ab") ;
#endif
/*
	}
*/
	if (!ReleaseMutex(*ghMutex))
	{
		retVal = GetLastError();
	}
#ifdef LOGGING
	fprintf(logfile,"Done\n") ;
	fclose(logfile) ;
#endif	
	return(retVal) ;
}
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
	extern "C" {
#endif
int ShutdownSharedMem(/* HANDLE *hMapFile, HANDLE *ghMutex,  int **pQueue*/)
{
	Privilege(TEXT("SeLockMemoryPrivilege"), FALSE);
	CloseHandle(*ghMutex);
	UnmapViewOfFile(*pQueue);
	CloseHandle(*hMapFile);
	return 0;
}
#ifdef __cplusplus
}
#endif
